import invariant from 'invariant';
import isEqual from 'lodash.isequal';

import { SYNC_STATE } from './constants';

const noop = _ => _;

export default function persistentReducer(
  reducer,
  reducerName,
  options = { }) {
  invariant(reducer, 'reducer is not defined.');
  invariant(reducerName, 'reducerName is not defined.');
  const { mapState = noop, stateIsEqual = isEqual } = options;
  return (state, action) => {
    if (action.type === SYNC_STATE && action.payload && action.payload.name === reducerName) {
      const mappedState = mapState(action.payload.state, state);
      if (!stateIsEqual(mappedState, state)) {
        return reducer(mappedState, action);
      }
    }
    return reducer(state, action);
  };
}
