import invariant from 'invariant';

import syncState from './syncState';

export default function persistentStore(db) {
  invariant(db, 'db is not defined.');

  return function storeEnhancer(storeCreator) {
    return function persistentStoreCreator(reducer, initialState, enhancer) {
      const store = storeCreator(reducer, initialState, enhancer);
      let isLoading = false;
      async function initDocs() {
        isLoading = true;
        const { rows } = await db.allDocs({ include_docs: true });
        isLoading = false;
        for (let i = 0, len = rows.length; i < len; i += 1) {
          const { doc } = rows[i];
          store.dispatch(syncState(doc));
        }
      }
      function replaceReducer(nextReducer) {
        store.replaceReducer(nextReducer);
        if (!isLoading) {
          initDocs();
        }
      }
      initDocs();
      db.changes({ include_docs: true, live: true, since: 'now' })
        .on('change', ({ doc }) => store.dispatch(syncState(doc)));
      return { ...store, replaceReducer };
    };
  };
}
